import logging

from nile import deployments
from nile.common import is_alias
# from nile.core.account import Account
from nile.utils import normalize_number, hex_class_hash, hex_address

from nile_upgrades.common import declare_class, get_contract_abi


async def upgrade_proxy(
    nre,
    account,
    proxy_address_or_alias,
    contract_name,
    max_fee_declare_impl=None,
    max_fee_upgrade_proxy=None,
    standalone_mode=None
):
    """
    Upgrade a proxy to a different implementation contract.

    Returns a Nile Transaction instance representing the upgrade operation.

    `nre` - the `NileRuntimeEnvironment` object.

    `account` - the Account to use.

    `proxy_address_or_alias` - the proxy address or alias.

    `contract_name` - the name of the implementation contract to upgrade to.

    `max_fee_declare_impl` - Maximum fee for declaring the new implementation contract. Defaults to `None`.

    `max_fee_upgrade_proxy` - Maximum fee for upgrading the proxy to the new implementation. Defaults to `None`.
    """

    proxy_address = _load_deployment(proxy_address_or_alias, nre.network)

    # Declare new implementation
    impl_class_hash = await declare_class(nre.network, contract_name, account, max_fee_declare_impl)

    # Perform upgrade
    logging.info(f"⏭️  Upgrading proxy {hex_address(proxy_address)} to class hash {hex_class_hash(impl_class_hash)}")
    upgrade_tx = await account.send(
        proxy_address, "upgrade", calldata=[impl_class_hash], max_fee=max_fee_upgrade_proxy
    )

    # Update ABI in deployments file
    deployments.update_abi(
        proxy_address, get_contract_abi(contract_name), nre.network
    )

    return upgrade_tx


def _load_deployment(proxy_address_or_alias, network):
    ids = None
    if not is_alias(proxy_address_or_alias):
        ids = deployments.load(normalize_number(proxy_address_or_alias), network)
    else:
        ids = deployments.load(proxy_address_or_alias, network)

    id = next(ids, None)
    if id is None:
        raise Exception(
            f"Deployment with address or alias {_normalize_string(proxy_address_or_alias)} not found"
        )
    if next(ids, None) is not None:
        raise Exception(
            f"Multiple deployments found with address or alias {_normalize_string(proxy_address_or_alias)}"
        )

    return id[0]


def _normalize_string(proxy_address_or_alias):
    identifier = proxy_address_or_alias
    if type(identifier) is int:
        identifier = hex_address(identifier)
    return identifier



import logging
import os

from starkware.starknet.compiler.compile import get_selector_from_name

from nile_upgrades.common import declare_class, get_contract_abi


async def deploy_proxy(
    nre,
    account,
    contract_name,
    salt,
    unique,
    initializer_args,
    initializer='initializer',
    alias=None,
    max_fee_declare_impl=None,
    max_fee_declare_proxy=None,
    max_fee_deploy_proxy=None,
    standalone_mode=None
):
    """
    Deploy an upgradeable proxy for an implementation contract.

    Returns a Nile Transaction instance representing the proxy deployment.

    `nre` - the `NileRuntimeEnvironment` object.

    `account` - the Account to use.

    `contract_name` - the name of the implementation contract.

    `salt` and `unique` - UDC specific arguments for proxy address generation.

    `initializer_args` - array of arguments for the initializer function.

    `initializer` - initializer function name. Defaults to `'initializer'`.

    `alias` - Unique identifier for your proxy. Defaults to `None`.

    `max_fee_declare_impl` - Maximum fee for declaring the implementation contract. Defaults to `None`.

    `max_fee_declare_proxy` - Maximum fee for declaring the proxy contract. Defaults to `None`.

    `max_fee_deploy_proxy` - Maximum fee for deploying the proxy contract. Defaults to `None`.
    """

    # Declare implementation
    impl_class_hash = await declare_class(nre.network, contract_name, account, max_fee_declare_impl)

    # Declare proxy
    await declare_class(nre.network, "Proxy", account, max_fee_declare_proxy, overriding_path=_get_proxy_artifact_path())

    # Deploy proxy
    logging.debug(f"Deploying proxy...")
    selector = get_selector_from_name(initializer)
    deploy_tx = await account.deploy_contract(
        "Proxy",
        salt=salt,
        unique=unique,
        calldata=[impl_class_hash, selector, len(initializer_args), *initializer_args],
        max_fee=max_fee_deploy_proxy,
        alias=alias,
        overriding_path=_get_proxy_artifact_path(),
        abi=get_contract_abi(contract_name),
    )

    return deploy_tx


def _get_proxy_artifact_path():
    package = os.path.dirname(os.path.realpath(__file__))
    return (f"{package}/artifacts", f"{package}/artifacts/abis")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nile_upgrades']

package_data = \
{'': ['*'], 'nile_upgrades': ['artifacts/*']}

install_requires = \
['click>=8.0.4,<9.0.0']

entry_points = \
{'console_scripts': ['compile = compile_proxy:main'],
 'nile_plugins.nre': ['deploy_proxy = nile_upgrades.deploy_proxy.deploy_proxy',
                      'upgrade_proxy = '
                      'nile_upgrades.upgrade_proxy.upgrade_proxy']}

setup_kwargs = {
    'name': 'nile-upgrades',
    'version': '0.0.1',
    'description': 'Nile plugin to deploy and manage upgradeable smart contracts on StarkNet',
    'long_description': '# OpenZeppelin Nile Upgrades\n\nPlugin for [Nile](https://github.com/OpenZeppelin/nile) to deploy and manage [upgradeable smart contracts](https://docs.openzeppelin.com/contracts-cairo/proxies) on StarkNet.\n\n> ## ⚠️ WARNING! ⚠️\n>\n> **This plugin does not currently validate contracts for upgrade safety.**\n>\n> This repo contains highly experimental code.\n> Expect rapid iteration.\n> **Use at your own risk.**\n\n## Installation\n\n```\npip install nile-upgrades\n```\n\n## Usage\n\nRun the following functions from scripts with the `NileRuntimeEnvironment`.\n\n### `deploy_proxy`\n\nDeploy an upgradeable proxy for an implementation contract.\n\nReturns a Nile Transaction instance representing the proxy deployment.\n\n```\nasync def deploy_proxy(\n    nre,\n    account,\n    contract_name,\n    salt,\n    unique,\n    initializer_args,\n    initializer=\'initializer\',\n    alias=None,\n    max_fee_declare_impl=None,\n    max_fee_declare_proxy=None,\n    max_fee_deploy_proxy=None,\n)\n```\n\n- `nre` - the `NileRuntimeEnvironment` object.\n\n- `account` - the Account to use.\n\n- `contract_name` - the name of the implementation contract.\n\n- `salt` and `unique` - UDC specific arguments for proxy address generation.\n\n- `initializer_args` - array of arguments for the initializer function.\n\n- `initializer` - initializer function name. Defaults to `\'initializer\'`.\n\n- `alias` - Unique identifier for your proxy. Defaults to `None`.\n\n- `max_fee_declare_impl` - Maximum fee for declaring the implementation contract. Defaults to `None`.\n\n- `max_fee_declare_proxy` - Maximum fee for declaring the proxy contract. Defaults to `None`.\n\n- `max_fee_deploy_proxy` - Maximum fee for deploying the proxy contract. Defaults to `None`.\n\nExample usage:\n```\ntx = await nre.deploy_proxy(nre, account, "my_contract_v1", 123, True, ["arg for initializer"])\ntx_status, proxy_address, abi = await tx.execute(watch_mode="track")\n```\n\n### `upgrade_proxy`  \n\nUpgrade a proxy to a different implementation contract.\n\nReturns a Nile Transaction instance representing the upgrade operation.\n\n```\nasync def upgrade_proxy(\n    nre,\n    account,\n    proxy_address_or_alias,\n    contract_name,\n    max_fee_declare_impl=None,\n    max_fee_upgrade_proxy=None,\n)\n```\n\n- `nre` - the `NileRuntimeEnvironment` object.\n\n- `account` - the Account to use.\n\n- `proxy_address_or_alias` - the proxy address or alias.\n\n- `contract_name` - the name of the implementation contract to upgrade to.\n\n- `max_fee_declare_impl` - Maximum fee for declaring the new implementation contract. Defaults to `None`.\n\n- `max_fee_upgrade_proxy` - Maximum fee for upgrading the proxy to the new implementation. Defaults to `None`.\n\nExample usage:\n```\ntx = await nre.upgrade_proxy(nre, account, proxy_address, "my_contract_v2")\ntx_status = await tx.execute(watch_mode="track")\n```\n\n## Contribute\n\n### Setup\n\n#### Using the latest Nile release supported by this plugin\n\n1. Install [Poetry](https://python-poetry.org/docs/#installation)\n2. Clone this project.\n3. From this project\'s root, create a virtualenv, activate it, and install dependencies:\n```\npython3.9 -m venv env\nsource env/bin/activate\npoetry install\npip install -e .\npoetry run compile\n```\n\n**or**\n\n#### Using current Nile source code\n\n1. Install [Poetry](https://python-poetry.org/docs/#installation)\n2. Clone https://github.com/OpenZeppelin/nile\n3. Clone this project.\n4. From this project\'s root, create a virtualenv, activate it, and install dependencies:\n```\npython3.9 -m venv env\nsource env/bin/activate\npoetry install\npip install -e <your_path_to_nile_repo_from_step_2>\npip install -e .\npoetry run compile\n```\n\n### Testing\n\n`poetry run pytest tests`\n',
    'author': 'Eric Lau',
    'author_email': 'ericglau@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/OpenZeppelin/openzeppelin-nile-upgrades',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)



import logging

from nile.common import get_class_hash, ABIS_DIRECTORY
from nile.deployments import class_hash_exists
from nile.utils import hex_class_hash


async def declare_class(network, contract_name, account, max_fee, overriding_path=None):
    """
    Declare a contract class and waits until the transaction completes.

    Returns the declared class hash in decimal format.
    """
    logging.debug(f"Declaring contract class {contract_name}...")
    class_hash = get_class_hash(contract_name=contract_name, overriding_path=overriding_path)
    padded_hash = hex_class_hash(class_hash)
    if class_hash_exists(class_hash, network):
        logging.debug(f"Contract class with hash {padded_hash} already exists")
    else:
        tx = await account.declare(contract_name, max_fee=max_fee, overriding_path=overriding_path)
        tx_status, declared_hash = await tx.execute(watch_mode="track")

        if tx_status.status.is_rejected:
            raise Exception(f"Could not declare contract class. Transaction rejected.", tx_status.error_message)

        if padded_hash != declared_hash:
            raise Exception(
                f"Declared hash {declared_hash} does not match expected hash {padded_hash}"
            )

        logging.debug(f"Contract class declared with hash {padded_hash}")

    return class_hash


def get_contract_abi(contract_name):
    return f"{ABIS_DIRECTORY}/{contract_name}.json"
